@@ -20,4 +20,5 @@
 #include "block.h"
 
 int dellssd_write(struct block_device *device, enum ibpi_pattern ibpi);
-char *dellssd_get_path(const char *path, const char *cntrl_path);
+char *dellssd_get_path(const char *cntrl_path);
+int get_dell_server_type(void);
