@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2021 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -47,7 +47,10 @@ struct enclosure_device {
    */
 	char *dev_path;
 
-	struct ses_pages *ses_pages;
+	struct ses_pages ses_pages;
+
+	struct ses_slot *slots;
+	int slots_count;
 };
 
 /**
@@ -78,4 +81,6 @@ struct enclosure_device *enclosure_device_init(const char *path);
  */
 void enclosure_device_fini(struct enclosure_device *enclosure);
 
+int enclosure_open(const struct enclosure_device *enclosure);
+
 #endif				/* _ENCLOSURE_H_INCLUDED_ */
