@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2021 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -17,6 +17,7 @@
  *
  */
 
+#include <config_ac.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -55,7 +56,6 @@
 #include "sysfs.h"
 #include "udev.h"
 #include "utils.h"
-#include "version.h"
 #include "vmdssd.h"
 
 /**
@@ -84,6 +84,14 @@ static sig_atomic_t terminate;
  */
 static char *ledmon_conf_path;
 
+/**
+ * @brief Boolean flag whether to run foreground or not.
+ *
+ * This flag is turned on with --foreground option. Primary use of this option
+ * is to use it in systemd service file.
+ */
+static int foreground;
+
 /**
  * @brief Name of IBPI patterns.
  *
@@ -110,19 +118,18 @@ const char *ibpi_str[] = {
  * Internal variable of monitor service. It is the pattern used to print out
  * information about the version of monitor service.
  */
-static char *ledmon_version = "Intel(R) Enclosure LED Monitor Service %d.%d\n"
-			      "Copyright (C) 2009-2018 Intel Corporation.\n";
+static char *ledmon_version = "Intel(R) Enclosure LED Monitor Service %s %s\n"
+			      "Copyright (C) 2009-2021 Intel Corporation.\n";
 
 /**
  * Internal variable of monitor service. It is used to help parse command line
  * short options.
  */
-static char *shortopt = "t:c:hvl:";
+static char *shortopt;
 
-/**
- * Internal enumeration type. It is used to help parse command line arguments.
- */
-enum longopt {
+struct option *longopt;
+
+static int possible_params[] = {
 	OPT_ALL,
 	OPT_CONFIG,
 	OPT_DEBUG,
@@ -135,27 +142,11 @@ enum longopt {
 	OPT_VERSION,
 	OPT_WARNING,
 	OPT_LOG_LEVEL,
+	OPT_FOREGROUND,
 };
 
-/**
- * Internal array with option tokens. It is used to help parse command line
- * long options.
- */
-static struct option longopt[] = {
-	[OPT_ALL]      = {"all", no_argument, NULL, '\0'},
-	[OPT_CONFIG]   = {"config", required_argument, NULL, 'c'},
-	[OPT_DEBUG]    = {"debug", no_argument, NULL, '\0'},
-	[OPT_ERROR]    = {"error", no_argument, NULL, '\0'},
-	[OPT_HELP]     = {"help", no_argument, NULL, 'h'},
-	[OPT_INFO]     = {"info", no_argument, NULL, '\0'},
-	[OPT_INTERVAL] = {"interval", required_argument, NULL, 't'},
-	[OPT_LOG]      = {"log", required_argument, NULL, 'l'},
-	[OPT_QUIET]    = {"quiet", no_argument, NULL, '\0'},
-	[OPT_VERSION]  = {"version", no_argument, NULL, 'v'},
-	[OPT_WARNING]  = {"warning", no_argument, NULL, '\0'},
-	[OPT_LOG_LEVEL] = {"log-level", required_argument, NULL, '\0'},
-			 {NULL, no_argument, NULL, '\0'}
-};
+static int possible_params_size = sizeof(possible_params)
+		/ sizeof(possible_params[0]);
 
 /**
  * @brief Monitor service finalize function.
@@ -165,17 +156,17 @@ static struct option longopt[] = {
  * file and syslog. The function is registered as on_exit() handler.
  *
  * @param[in]     status          The function ignores this parameter.
- * @param[in]     progname        The name of the binary file. This argument
+ * @param[in]     program_name    The name of the binary file. This argument
  *                                is passed via on_exit() function.
  *
  * @return The function does not return a value.
  */
-static void _ledmon_fini(int __attribute__ ((unused)) status, void *progname)
+static void _ledmon_fini(int __attribute__ ((unused)) status, void *program_name)
 {
 	sysfs_reset();
 	list_erase(&ledmon_block_list);
 	log_close();
-	pidfile_remove(progname);
+	pidfile_remove(program_name);
 }
 
 /**
@@ -187,19 +178,31 @@ static void _ledmon_fini(int __attribute__ ((unused)) status, void *progname)
  *
  * @param[in]     status            Status given in the last call to exit()
  *                                  function.
- * @param[in]     ignore            Pointer to placeholder where ignore flag is
- *                                  stored. If flag is set 0 then parent process
- *                                  is exiting, otherwise a child is exiting.
- *                                  This argument must not be NULL pointer.
+ * @param[in]     arg               Argument passed to on_exit().
  *
  * @return The function does not return a value.
  */
-static void _ledmon_status(int status, void *ignore)
+static void _ledmon_status(int status, void *arg)
 {
-	if (*((int *)ignore) != 0)
-		log_info("exit status is %s.", strstatus(status));
-	else if (status != STATUS_SUCCESS)
-		log_error("parent exit status is %s.", strstatus(status));
+	int log_level;
+	char message[4096];
+	int ignore = *((int *)arg);
+
+	if (ignore)
+		return;
+
+	if (status == STATUS_SUCCESS)
+		log_level = LOG_LEVEL_INFO;
+	else
+		log_level = LOG_LEVEL_ERROR;
+
+	snprintf(message, sizeof(message), "exit status is %s.",
+		 strstatus(status));
+
+	if (get_log_fd() >= 0)
+		_log(log_level, message);
+	else
+		syslog(log_level_infos[log_level].priority, "%s", message);
 }
 
 /**
@@ -213,7 +216,7 @@ static void _ledmon_status(int status, void *ignore)
  */
 static void _ledmon_version(void)
 {
-	printf(ledmon_version, VERSION_MAJOR, VERSION_MINOR);
+	printf(ledmon_version, PACKAGE_VERSION, BUILD_LABEL);
 	printf("\nThis is free software; see the source for copying conditions."
 	       " There is NO warranty;\nnot even for MERCHANTABILITY or FITNESS"
 	       " FOR A PARTICULAR PURPOSE.\n\n");
@@ -231,7 +234,7 @@ static void _ledmon_version(void)
  */
 static void _ledmon_help(void)
 {
-	printf(ledmon_version, VERSION_MAJOR, VERSION_MINOR);
+	printf(ledmon_version, PACKAGE_VERSION, BUILD_LABEL);
 	printf("\nUsage: %s [OPTIONS]\n\n", progname);
 	printf("Mandatory arguments for long options are mandatory for short "
 	       "options, too.\n\n");
@@ -244,6 +247,8 @@ static void _ledmon_help(void)
 			  "Use local log file instead /var/log/ledmon.log");
 	print_opt("--log-level=VALUE", "-l VALUE",
 			  "Allows user to set ledmon verbose level in logs.");
+	print_opt("--foreground", "",
+			  "Do not run as daemon.");
 	print_opt("--help", "-h", "Displays this help text.");
 	print_opt("--version", "-v",
 			  "Displays version and license information.");
@@ -269,7 +274,7 @@ static status_t _set_config_path(char **conf_path, const char *path)
 
 	if (*conf_path)
 		free(*conf_path);
-	*conf_path = strdup(path);
+	*conf_path = str_dup(path);
 
 	return STATUS_SUCCESS;
 }
@@ -287,7 +292,12 @@ static status_t _set_config_path(char **conf_path, const char *path)
  */
 static status_t _set_sleep_interval(const char *optarg)
 {
-	conf.scan_interval = atoi(optarg);
+	errno = 0;
+	conf.scan_interval = strtol(optarg, NULL, 10);
+	if (errno != 0) {
+		log_error("Cannot parse sleep interval");
+		return STATUS_CMDLINE_ERROR;
+	}
 	if (conf.scan_interval < LEDMON_MIN_SLEEP_INTERVAL) {
 		log_warning("sleep interval too small... using default.");
 		conf.scan_interval = LEDMON_DEF_SLEEP_INTERVAL;
@@ -295,73 +305,6 @@ static status_t _set_sleep_interval(const char *optarg)
 	return STATUS_SUCCESS;
 }
 
-/**
- * @brief Sets verbose variable to given level.
- *
- * This is internal function of monitor service. The function maps given level
- * to the value from verbose_level enum and sets verbose value to ledmon
- * configuration.
- *
- * @param[in]      log_level     required new log_level.
- *
- * @return STATUS_SUCCESS if successful, otherwise a valid status_t status code.
- */
-static status_t _set_verbose_level(int log_level)
-{
-	int new_verbose = -1;
-
-	switch (log_level) {
-	case OPT_ALL:
-		new_verbose = LOG_LEVEL_ALL;
-		break;
-	case OPT_DEBUG:
-		new_verbose = LOG_LEVEL_DEBUG;
-		break;
-	case OPT_ERROR:
-		new_verbose = LOG_LEVEL_ERROR;
-		break;
-	case OPT_INFO:
-		new_verbose = LOG_LEVEL_INFO;
-		break;
-	case OPT_QUIET:
-		new_verbose = LOG_LEVEL_QUIET;
-		break;
-	case OPT_WARNING:
-		new_verbose = LOG_LEVEL_WARNING;
-		break;
-	}
-	if (new_verbose != -1) {
-		conf.log_level = new_verbose;
-		return STATUS_SUCCESS;
-	}
-	return STATUS_CMDLINE_ERROR;
-}
-
-/**
- * @brief Gets id for given CLI option which corresponds to value from longopt
- * table.
- *
- * This is internal function of monitor service. The function maps given string
- * to the value from longopt enum and returns id of matched element. Generic
- * parameters allow to use this function for any CLI options-table which bases
- * on option struct.
- *
- * @param[in]     optarg          String containing value given by user in CLI.
- * @param[in]     longopt         Table of allowed CLI options.
- *
- * @return integer id if successful, otherwise a -1.
- */
-static int _get_option_id(const char *optarg, struct option longopt[])
-{
-	struct option *i = longopt;
-
-	while (i->name != NULL) {
-		if (strcmp(i->name, optarg) == 0)
-			return i-longopt;
-		i++;
-	}
-	return -1;
-}
 
 /**
  * @brief Reads config file path and checks if command line input contains
@@ -402,7 +345,6 @@ static status_t _cmdline_parse_non_daemonise(int argc, char *argv[])
 
 	return status;
 }
-
 /**
  * @brief Command line interface handler function.
  *
@@ -427,18 +369,22 @@ static status_t _cmdline_parse(int argc, char *argv[])
 		if (opt == 'c')
 			continue;
 		switch (opt) {
-		int new_log_level;
+		int log_level;
 		case 0:
-			switch (opt_index) {
+			switch (get_option_id(longopt[opt_index].name)) {
 			case OPT_LOG_LEVEL:
-				new_log_level = _get_option_id(optarg, longopt);
-				if (new_log_level != -1)
-					status = _set_verbose_level(new_log_level);
+				log_level = get_option_id(optarg);
+				if (log_level != -1)
+					status = set_verbose_level(log_level);
 				else
 					status = STATUS_CMDLINE_ERROR;
 				break;
+			case OPT_FOREGROUND:
+				foreground = 1;
+				break;
 			default:
-				status = _set_verbose_level(opt);
+				status = set_verbose_level(
+						possible_params[opt_index]);
 			}
 			break;
 		case 'l':
@@ -546,7 +492,7 @@ static void _ledmon_wait(int seconds)
 			FD_SET(udev_fd, &rdfds);
 
 		res = pselect(max_fd, &rdfds, NULL, &exfds, &timeout, &sigset);
-		if (!FD_ISSET(udev_fd, &rdfds) ||
+		if (terminate || !FD_ISSET(udev_fd, &rdfds) ||
 		    handle_udev_event(&ledmon_block_list) <= 0)
 			break;
 	} while (res > 0);
@@ -646,17 +592,7 @@ static void _handle_fail_state(struct block_device *block,
 				 * Trasitions other than raid 0 migration.
 				 * Like reshape, volume stopping etc.
 				 */
-				if (!temp_raid_device) {
-					if (block->raid_dev->sync_action !=
-						RAID_ACTION_RESHAPE)
-						/*
-						 * Fail state should be
-						 * released,because disk is in
-						 * reshape process.
-						 */
-						temp->ibpi =
-							IBPI_PATTERN_HOTSPARE;
-				} else {
+				if (temp_raid_device) {
 					/*
 					 * Drive is removed from volume,
 					 * but still exist in container. This
@@ -733,22 +669,22 @@ static void _add_block(struct block_device *block)
 
 		if (ibpi != temp->ibpi && ibpi <= IBPI_PATTERN_REMOVED) {
 			log_info("CHANGE %s: from '%s' to '%s'.",
-				 temp->sysfs_path, ibpi_str[ibpi],
-				 ibpi_str[temp->ibpi]);
+				 temp->sysfs_path, ibpi2str(ibpi),
+				 ibpi2str(temp->ibpi));
 		}
 		/* Check if name of the device changed.*/
 		if (strcmp(temp->sysfs_path, block->sysfs_path)) {
 			log_info("NAME CHANGED %s to %s",
 				 temp->sysfs_path, block->sysfs_path);
 			free(temp->sysfs_path);
-			temp->sysfs_path = strdup(block->sysfs_path);
+			temp->sysfs_path = str_dup(block->sysfs_path);
 		}
 	} else {
 		/* Device not found, it's a new one! */
 		temp = block_device_duplicate(block);
 		if (temp != NULL) {
 			log_info("NEW %s: state '%s'.", temp->sysfs_path,
-				 ibpi_str[temp->ibpi]);
+				 ibpi2str(temp->ibpi));
 			list_append(&ledmon_block_list, temp);
 		}
 	}
@@ -780,8 +716,8 @@ static void _send_msg(struct block_device *block)
 	    block->ibpi == IBPI_PATTERN_REMOVED) {
 		if (block->ibpi != IBPI_PATTERN_FAILED_DRIVE) {
 			log_info("CHANGE %s: from '%s' to '%s'.",
-				 block->sysfs_path, ibpi_str[block->ibpi],
-				 ibpi_str[IBPI_PATTERN_FAILED_DRIVE]);
+				 block->sysfs_path, ibpi2str(block->ibpi),
+				 ibpi2str(IBPI_PATTERN_FAILED_DRIVE));
 			block->ibpi = IBPI_PATTERN_FAILED_DRIVE;
 		} else {
 			char *host = strstr(block->sysfs_path, "host");
@@ -890,7 +826,7 @@ static status_t _init_ledmon_conf(void)
 	conf.blink_on_init = 1;
 	conf.blink_on_migration = 1;
 	conf.rebuild_blink_on_all = 0;
-	conf.raid_memebers_only = 0;
+	conf.raid_members_only = 0;
 	conf.log_level = LOG_LEVEL_WARNING;
 	conf.scan_interval = LEDMON_DEF_SLEEP_INTERVAL;
 	list_init(&conf.cntrls_whitelist, NULL);
@@ -920,14 +856,20 @@ static void _close_parent_fds(void)
 int main(int argc, char *argv[])
 {
 	status_t status = STATUS_SUCCESS;
+	int ignore = 0;
 
+	setup_options(&longopt, &shortopt, possible_params,
+			possible_params_size);
 	set_invocation_name(argv[0]);
 	openlog(progname, LOG_PID | LOG_PERROR, LOG_DAEMON);
 
+	if (on_exit(_ledmon_status, &ignore))
+		return STATUS_ONEXIT_ERROR;
+
 	if (_cmdline_parse_non_daemonise(argc, argv) != STATUS_SUCCESS)
 		return STATUS_CMDLINE_ERROR;
 
-	if (getuid() != 0) {
+	if (geteuid() != 0) {
 		fprintf(stderr, "Only root can run this application.\n");
 		return STATUS_NOT_A_PRIVILEGED_USER;
 	}
@@ -936,43 +878,50 @@ int main(int argc, char *argv[])
 	if (status != STATUS_SUCCESS)
 		return status;
 
-	if (on_exit(_ledmon_status, &terminate))
-		return STATUS_ONEXIT_ERROR;
-
 	status = ledmon_read_config(ledmon_conf_path);
 	if (status != STATUS_SUCCESS)
 		return status;
 
-	if(conf.log_path)
-		set_log_path(conf.log_path);
-
 	if (_cmdline_parse(argc, argv) != STATUS_SUCCESS)
 		return STATUS_CMDLINE_ERROR;
 
+	ledmon_write_shared_conf();
+
+	if (log_open(conf.log_path) != STATUS_SUCCESS)
+		return STATUS_LOG_FILE_ERROR;
+
+	free(shortopt);
+	free(longopt);
 	if (pidfile_check(progname, NULL) == 0) {
 		log_warning("daemon is running...");
 		return STATUS_LEDMON_RUNNING;
 	}
+	if (!foreground) {
+		pid_t pid = fork();
 
-	pid_t pid = fork();
-	if (pid < 0) {
-		log_debug("main(): fork() failed (errno=%d).", errno);
-		exit(EXIT_FAILURE);
-	}
-	if (pid > 0)
-		exit(EXIT_SUCCESS);
+		if (pid < 0) {
+			log_debug("main(): fork() failed (errno=%d).", errno);
+			exit(EXIT_FAILURE);
+		}
+		if (pid > 0) {
+			ignore = 1; /* parent: don't print exit status */
+			exit(EXIT_SUCCESS);
+		}
 
-	pid_t sid = setsid();
-	if (sid < 0) {
-		log_debug("main(): setsid() failed (errno=%d).", errno);
-		exit(EXIT_FAILURE);
+		pid_t sid = setsid();
+
+		if (sid < 0) {
+			log_debug("main(): setsid() failed (errno=%d).", errno);
+			exit(EXIT_FAILURE);
+		}
+
+		_close_parent_fds();
+
+		int t = open("/dev/null", O_RDWR);
+		UNUSED(dup(t));
+		UNUSED(dup(t));
 	}
 
-	_close_parent_fds();
-
-	int t = open("/dev/null", O_RDWR);
-	dup(t);
-	dup(t);
 	umask(027);
 
 	if (chdir("/") < 0) {
@@ -1002,6 +951,7 @@ int main(int argc, char *argv[])
 			_invalidate_dev(device);
 		sysfs_reset();
 	}
+	ledmon_remove_shared_conf();
 	stop_udev_monitor();
 	exit(EXIT_SUCCESS);
 }
