@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2017 Intel Corporation.
+ * Copyright (C) 2009-2021 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -20,6 +20,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <signal.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,6 +37,9 @@
 #include "status.h"
 #include "utils.h"
 
+#define RUN_DIR "/var/run/"
+#define PID ".pid"
+
 /**
  */
 status_t pidfile_create(const char *name)
@@ -43,9 +47,7 @@ status_t pidfile_create(const char *name)
 	char buf[PATH_MAX];
 	int fd, count;
 
-	str_cpy(buf, "/var/run/", PATH_MAX);
-	str_cat(buf, name, PATH_MAX);
-	str_cat(buf, ".pid", PATH_MAX);
+	snprintf(buf, sizeof(buf), "%s/%s%s", RUN_DIR, name, PID);
 
 	fd = open(buf, O_WRONLY | O_CREAT, 0640);
 	if (fd < 0)
@@ -66,10 +68,7 @@ int pidfile_remove(const char *name)
 {
 	char buf[PATH_MAX];
 
-	str_cpy(buf, "/var/run/", PATH_MAX);
-	str_cat(buf, name, PATH_MAX);
-	str_cat(buf, ".pid", PATH_MAX);
-
+	snprintf(buf, sizeof(buf), "%s/%s%s", RUN_DIR, name, PID);
 	return unlink(buf);
 }
 
@@ -81,9 +80,9 @@ int pidfile_remove(const char *name)
  */
 int ping_proc(pid_t pid)
 {
-	char buf[PATH_MAX];
-	sprintf(buf, "kill -n 10 %d 2>/dev/null", pid);
-	if (system(buf) == 0)
+	if (pid <= 0)
+		return STATUS_INVALID_PATH;
+	if (kill(pid, 1) == 0)
 		return STATUS_SUCCESS;
 	return STATUS_INVALID_PATH;
 }
@@ -95,14 +94,12 @@ status_t pidfile_check(const char *name, pid_t *pid)
 	char path[PATH_MAX], *p;
 	pid_t tp;
 
-	str_cpy(path, "/var/run/", PATH_MAX);
-	str_cat(path, name, PATH_MAX);
-	str_cat(path, ".pid", PATH_MAX);
+	snprintf(path, sizeof(path), "%s/%s%s", RUN_DIR, name, PID);
 
 	p = buf_read(path);
 	if (p == NULL)
 		return STATUS_INVALID_PATH;
-	tp = atoi(p);
+	tp = strtol(p, NULL, 10);
 	if (pid)
 		*pid = tp;
 	free(p);
