@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2021 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -82,7 +82,7 @@ static unsigned int _get_slot(const char *path)
 	char *p = get_text(path, "slot");
 	if (p) {
 		if (strcmp(p, "none") != 0)
-			result = atoi(p);
+			result = strtol(p, NULL, 10);
 		free(p);
 	}
 	return result;
@@ -96,8 +96,7 @@ static struct block_device *_get_block(const char *path, struct list *block_list
 	char link[PATH_MAX];
 	struct block_device *device;
 
-	str_cpy(temp, path, PATH_MAX);
-	str_cat(temp, "/block", PATH_MAX);
+	snprintf(temp, sizeof(temp), "%s/block", path);
 
 	if (!realpath(temp, link))
 		return NULL;
