@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2011-2017 Intel Corporation.
+ * Copyright (C) 2011-2021 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -63,21 +63,6 @@ struct gpio_tx_register_byte {
 	unsigned char activity:3;
 } __attribute__ ((__packed__));
 
-/**
- * @brief Sends message to SES processor of an enclosure.
- *
- * This function send a message to an enclosure in order to control LEDs of
- * the given slot/component. It uses interface of ENCLOSURE kernel module to
- * control LEDs.
- *
- * @param[in]      device         Path to an enclosure device in sysfs.
- * @param[in]      ibpi           IBPI pattern to visualize.
- *
- * @return Number of characters written if successful or -1 in case of error
- *         and errno is set to appropriate error code.
- */
-int scsi_ses_write(struct block_device *device, enum ibpi_pattern ibpi);
-
 /**
  * @brief Write message to outbound raw byte stream buffer.
  *
