@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2019 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -125,7 +125,7 @@ static enum device_type _get_device_type(const char *path)
 	if (p != NULL) {
 		if (strlen(p) > 0) {
 			if (strncmp(p, "external:", 9) == 0) {
-				if (p[9] == '/')
+				if (p[9] == '/' || p[9] == '-')
 					result = DEVICE_TYPE_VOLUME;
 				else
 					result = DEVICE_TYPE_CONTAINER;
@@ -156,8 +156,7 @@ static void _get_id(const char *path, struct device_id *d_id)
 {
 	char temp[PATH_MAX];
 
-	str_cpy(temp, path, PATH_MAX);
-	str_cat(temp, "/dev", PATH_MAX);
+	snprintf(temp, sizeof(temp), "%s/dev", path);
 	get_id(temp, d_id);
 }
 
@@ -179,7 +178,7 @@ static void _slave_vol_add(const char *path, struct raid_device *raid)
 {
 	struct slave_device *device;
 
-	char *t = rindex(path, '/');
+	char *t = strrchr(path, '/');
 	if (strncmp(t + 1, "dev-", 4) == 0) {
 		device = slave_device_init(path, &sysfs_block_list);
 		if (device) {
@@ -240,7 +239,7 @@ static void _slave_cnt_add(const char *path, struct raid_device *raid)
 {
 	struct slave_device *device;
 
-	char *t = rindex(path, '/');
+	char *t = strrchr(path, '/');
 	if (strncmp(t + 1, "dev-", 4) == 0) {
 		device = slave_device_init(path, &sysfs_block_list);
 		if (device) {
@@ -259,8 +258,7 @@ static void _link_raid_device(struct raid_device *device, enum device_type type)
 	char temp[PATH_MAX];
 	struct list dir;
 
-	str_cpy(temp, device->sysfs_path, PATH_MAX);
-	str_cat(temp, "/md", PATH_MAX - 1);
+	snprintf(temp, sizeof(temp), "%s/md", device->sysfs_path);
 
 	if (scan_dir(temp, &dir) == 0) {
 		const char *dir_path;
@@ -423,7 +421,7 @@ static void _scan_slave(void)
 		_link_raid_device(device, DEVICE_TYPE_VOLUME);
 	list_for_each(&cntnr_list, device)
 		_link_raid_device(device, DEVICE_TYPE_CONTAINER);
-	if (conf.raid_memebers_only) {
+	if (conf.raid_members_only) {
 		struct node *node;
 
 		list_for_each_node(&sysfs_block_list, node) {
@@ -489,7 +487,7 @@ static void _set_block_state(struct block_device *block, enum ibpi_pattern ibpi)
 	char *debug_dev = strrchr(block->sysfs_path, '/');
 	debug_dev = debug_dev ? debug_dev + 1 : block->sysfs_path;
 	log_debug("(%s): device: %s, state: %s", __func__, debug_dev,
-		  ibpi_str[ibpi]);
+		  ibpi2str(ibpi));
 	if (block->ibpi < ibpi)
 		block->ibpi = ibpi;
 }
@@ -561,11 +559,11 @@ static void _determine(struct slave_device *device)
 	}
 }
 
-static void _determine_slaves(struct list *slave_list)
+static void _determine_slaves(struct list *local_slave_list)
 {
 	struct slave_device *device;
 
-	list_for_each(slave_list, device)
+	list_for_each(local_slave_list, device)
 		_determine(device);
 }
 
@@ -663,10 +661,9 @@ int sysfs_check_driver(const char *path, const char *driver)
 	char driver_path[PATH_MAX];
 	char *link;
 	int found = 0;
-	str_cpy(buf, path, PATH_MAX);
-	str_cat(buf, "/driver", PATH_MAX - 1);
-	snprintf(driver_path, PATH_MAX - 1, "/%s", driver);
 
+	snprintf(buf, sizeof(buf), "%s/driver", path);
+	snprintf(driver_path, sizeof(driver_path), "/%s", driver);
 	link = realpath(buf, NULL);
 	if (link && strstr(link, driver_path))
 		found = 1;
