@@ -1,6 +1,6 @@
 /*
  * Intel(R) Enclosure LED Utilities
- * Copyright (C) 2009-2018 Intel Corporation.
+ * Copyright (C) 2009-2019 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -20,10 +20,20 @@
 #ifndef _UTILS_H_INCLUDED_
 #define _UTILS_H_INCLUDED_
 
+#include <getopt.h>
+#include "config_file.h"
 #include "stdlib.h"
 #include "stdint.h"
 #include "list.h"
 #include "status.h"
+#include "syslog.h"
+#include "ibpi.h"
+
+/**
+ * Value is intentionally unused.
+ * Macro to prevent compiler from raising unused result warning.
+ */
+#define UNUSED(x) do { if (x); } while (0)
 
 /**
  * Maximum number of bytes in temporary buffer. It is used for local variables.
@@ -44,6 +54,23 @@ struct device_id {
 	int major, minor;
 };
 
+struct log_level_info {
+	char prefix[10];
+	int priority;
+};
+
+/**
+ */
+#define PREFIX_DEBUG          "  DEBUG: "
+#define PREFIX_WARNING        "WARNING: "
+#define PREFIX_INFO           "   INFO: "
+#define PREFIX_ERROR          "  ERROR: "
+
+/**
+ * This array describes a log levels priorities with message prefix .
+ */
+extern struct log_level_info log_level_infos[];
+
 /**
  * This global variable holds the name of binary file an application has been
  * executed from.
@@ -175,7 +202,7 @@ int scan_dir(const char *path, struct list *result);
  *
  * @return Number of bytes written if successful, otherwise -1 for an error.
  */
-int buf_write(const char *path, const char *buf);
+ssize_t buf_write(const char *path, const char *buf);
 
 /**
  * @brief Reads the content of a text file.
@@ -231,57 +258,23 @@ int log_open(const char *path);
 void log_close(void);
 
 /**
- * @brief Logs an error message.
+ * @brief Logs an message with given loglevel.
  *
- * The function logs a message at error level of verbosity.
+ * The function logs a message at given level of verbosity.
  *
+ * @param[in]      loglevel       Level of verbosity for a message.
  * @param[in]      buf            Buffer containing format of a message.
  * @param[in]      ...            Additional arguments according to format of
  *                                a message.
  *
  * @return The function does not return a value.
  */
-void log_error(const char *buf, ...);
-
-/**
- * @brief Logs a debug message.
- *
- * The function logs a message at debug level of verbosity.
- *
- * @param[in]      buf            Buffer containing format of a message.
- * @param[in]      ...            Additional arguments according to format of
- *                                a message.
- *
- * @return The function does not return a value.
- */
-void log_debug(const char *buf, ...);
-
-/**
- * @brief Logs a warning message.
- *
- * The function logs a message at warning level of verbosity.
- *
- * @param[in]      buf            Buffer containing format of a message.
- * @param[in]      ...            Additional arguments according to format of
- *                                a message.
- *
- * @return The function does not return a value.
- */
-void log_warning(const char *buf, ...);
-
-/**
- * @brief Logs a information message.
- *
- * The function logs a message at info level of verbosity.
- *
- * @param[in]      buf            Buffer containing format of a message.
- * @param[in]      ...            Additional arguments according to format of
- *                                a message.
- *
- * @return The function does not return a value.
- */
-void log_info(const char *buf, ...);
+void _log(enum log_level_enum loglevel, const char *buf, ...);
 
+#define log_error(buf, ...)	_log(LOG_LEVEL_ERROR, buf, ##__VA_ARGS__)
+#define log_debug(buf, ...)	_log(LOG_LEVEL_DEBUG, buf, ##__VA_ARGS__)
+#define log_info(buf, ...)	_log(LOG_LEVEL_INFO, buf, ##__VA_ARGS__)
+#define log_warning(buf, ...)	_log(LOG_LEVEL_WARNING, buf, ##__VA_ARGS__)
 /**
  */
 void set_invocation_name(char *invocation_name);
@@ -314,25 +307,6 @@ char *str_cpy(char *dest, const char *src, size_t size);
  */
 char *str_dup(const char *src);
 
-/**
- * @brief Concatenates text buffers.
- *
- * This function appends source buffer to destination buffer. It is similar to
- * strncat() standard C function except it always returns null-terminated
- * buffer. Second difference is that function calculates itself the amount
- * of free space in destination buffer. If source does not fit in dest
- * then as many bytes are copied as can be fit in destination buffer minus 1
- * for null-character. Otherwise the source is copied to destination
- * including null-character.
- *
- * @param[in,out]  dest            Pointer to destination buffer.
- * @param[in]      src             Pointer to source buffer.
- * @param[in]      size            Capacity of destination buffer.
- *
- * @return Pointer to destination buffer even if function failed.
- */
-char *str_cat(char *dest, const char *src, size_t size);
-
 /**
  */
 char *truncate_path_component_rev(const char *path, int index);
@@ -360,4 +334,34 @@ void print_opt(const char *long_opt, const char *short_opt, const char *desc);
  */
 status_t set_log_path(const char *path);
 
+/**
+ * Internal enumeration type. It is used to help parse command line arguments.
+ */
+enum opt {
+	OPT_ALL,
+	OPT_CONFIG,
+	OPT_DEBUG,
+	OPT_ERROR,
+	OPT_HELP,
+	OPT_INFO,
+	OPT_INTERVAL,
+	OPT_LOG,
+	OPT_QUIET,
+	OPT_VERSION,
+	OPT_WARNING,
+	OPT_LOG_LEVEL,
+	OPT_LIST_CTRL,
+	OPT_LISTED_ONLY,
+	OPT_FOREGROUND,
+	OPT_NULL_ELEMENT
+};
+
+extern struct option longopt_all[];
+void setup_options(struct option **longopt, char **shortopt, int *options,
+			int options_nr);
+int get_option_id(const char *optarg);
+status_t set_verbose_level(int log_level);
+
+const char *ibpi2str(enum ibpi_pattern ibpi);
+
 #endif				/* _UTILS_H_INCLUDED_ */
